/* Wiiheadtrack-demo: object-mode
 * 
 * This program demonstrates the object-mode of the Wiiheadtrack-class, 
 * presenting a teapot in OpenGL. Use the makefile for compiling an running the 
 * program. You maybe need to adjust wiiht.setEyeDist(float, float) and
 * wiiht.setPosDist(float, float). Read the documentation for more information.
 * 
 * OpenGL must be installed on you system!
 * 
 * Press 'q' on your keyboard for closing the application.
 * 
 * author:			Daniel Wunderlich (d.wunderlich@stud.uni-heidelberg.de)
 * author:			Christian Rohrbeck (c.rohrbec@stud.uni-heidelberg.de)
 * version:			1.0
 * last modified:	2010-09-06
 * created:			2010-09-06
 * web:				http://pille.iwr.uni-heidelberg.de/~tracking01/
 */

#include <iostream>
#include <cstdlib>
#include <math.h>
#include <GL/glut.h>
#include "wiiheadtrack.h"

void init(void);
void display(void);
void reshape (int, int);
int main(int, char **);

Wiiheadtrack wiiht(WIIHT_MODE_OBJECT, 1);	// Create Wiiheadtrack-object

/* Idle-function */
void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glColor3f (1.0,0.0,0.0);
	
	/* Using Wiiheadtrack */
	wiiht.computePos();		// Computing the current position
	wiiht.printStatus();	// Print the current status to command-line
	
	float* posPointer = wiiht.getPos();	// Getting the current position
	float pos[3];
	pos[0] = *posPointer;
	pos[1] = *(++posPointer);
	pos[2] = *(++posPointer);
	
	gluLookAt(pos[0], pos[1], pos[2], 0,0,0, 0,1,0);	// Using the positions
	/* -------------------*/
	
	//glRotatef(90, 0, 1, 0);
	glutSolidTeapot(1.0);
	
	glutSwapBuffers();
}

void reshape (int w, int h)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45, 1.0 * w / h, 1, 1000);
   
  glViewport(0, 0, w, h);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0, 0, 100, 0, 0, 0, 0, 1, 0);	
}

void keyboard (unsigned char key, int x, int y) 
{ 
		
	switch (key) {
		case 'q':
		case 'Q':
			exit(EXIT_SUCCESS);
			break;
	}
}

void init(void)
{
  glClearColor (1.0, 1.0, 1.0, 0.0);
  
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  glEnable(GL_DEPTH_TEST);

  glEnable(GL_LINE_SMOOTH);
  glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glEnable(GL_COLOR_MATERIAL);
 }
 
int main(int argc,char** argv)
{	
	/* Setting up the Wiiheadtrack */
	wiiht.setScanTime(4);			// Configuring scan-time
	wiiht.connect();				// Connecting to the Wiimote(s)
	//wiiht.invertY();				// Maybe you need to invert the y-Axis
	wiiht.setSensitivity(40.0);		// Set the (x/y-)sensitivity
	wiiht.setEyeDist(730.0, 550.0);	// Set the eye-distance
	wiiht.setPosDist(6.0, 3.0);	// Set the position-distance (z-sensitivity)
	/* ----------------------- */
		
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB);
	glutInitWindowSize (800,800);
	glutInitWindowPosition (100, 100);
	glutCreateWindow ("Wiiheadtrack-Demo -- Object-Mode");
	init();
	
	glutKeyboardFunc(keyboard);
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
	glutIdleFunc(display);
	
	glutMainLoop();
	return 0;
}
