/* Wiiheadtrack-demo: room-mode
 * 
 * This program demonstrates the room-mode of the Wiiheadtrack-class using 
 * OpenGL. The scene is an imitation of Johnny Chung Lee's demo-room. Use the 
 * makefile for compiling an running the program. You maybe need to adjust 
 * wiiht.setEyeDist(float, float) and wiiht.setPosDist(float, float). Read the 
 * documentation for more information.
 * 
 * OpenGL must be installed on you system!
 * 
 * Press 'q' on your keyboard for closing the application.
 * 
 * author:			Christian Rohrbeck (c.rohrbec@stud.uni-heidelberg.de)
 * author:			Daniel Wunderlich (d.wunderlich@stud.uni-heidelberg.de)
 * version:			1.0
 * last modified:	2010-09-06
 * created:			2010-09-06
 * web:				http://pille.iwr.uni-heidelberg.de/~tracking01/
 */

#include <iostream>
#include <cstdlib>
#include <math.h>
#include <GL/glut.h>
#include "wiiheadtrack.h"

void init(void);
void display(void);
void reshape (int, int);
int main(int, char **);

Wiiheadtrack wiiht(WIIHT_MODE_ROOM, 1);

// room-settings
double roomDepth = 100.0;
double roomBeginning = 100.0;
double roomSize = 10.0;
double roomNrHLines = 10;
double roomNrVLines = 100;

// random numbers
double randNumbersX[200];
double randNumbersY[200];

void drawQuad(double x, double y, double z, double size) 
{
	double size2 = size/2;
	glBegin(GL_QUADS);
		glVertex3f(x-size2, y+size2, z);
		glVertex3f(x+size2, y+size2, z);
		glVertex3f(x+size2, y-size2, z);
		glVertex3f(x-size2, y-size2, z);
	glEnd();
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glLoadIdentity();
	
	glColor3f (1.0,0.0,0.0);
	
	/* Using Wiiheadtrack */
	wiiht.computePos();		// Computing the current position
	wiiht.printStatus();	// Print the current status to command-line
	
	float* posPointer = wiiht.getPos();	// Getting the current position
	float pos[3];
	pos[0] = *posPointer;
	pos[1] = *(++posPointer);
	pos[2] = *(++posPointer);
	/* -------------------*/
	
	gluLookAt(pos[0], pos[1], pos[2], pos[0],pos[1],pos[2]-1, 0,1,0);	// Using the positions
	
	
	//----------------------------
	// Room
	//----------------------------

	glColor3f(1.0, 1.0, 1.0);
	
	double linePos;
	
	glLineWidth(3.0);
	// Horizontal Lines
	for(linePos = -roomSize/2; linePos <= roomSize/2; linePos = linePos + roomSize/roomNrHLines) {
		// Lines right
		glBegin(GL_LINES);
			glVertex3f(roomSize/2, linePos, roomBeginning);
			glVertex3f(roomSize/2, linePos, -roomDepth);
		glEnd();

		// Lines left
		glBegin(GL_LINES);
			glVertex3f(-roomSize/2, linePos, roomBeginning);
			glVertex3f(-roomSize/2, linePos, -roomDepth);
		glEnd();

		// Lines bottom
		glBegin(GL_LINES);
			glVertex3f(-linePos, -roomSize/2, roomBeginning);
			glVertex3f(-linePos, -roomSize/2, -roomDepth);
		glEnd();

		// Lines top
		glBegin(GL_LINES);
			glVertex3f(linePos, roomSize/2, roomBeginning);
			glVertex3f(linePos, roomSize/2, -roomDepth);
		glEnd();
	}

	// Vertical Lines and Targets
	int countX = 0;
	int countY = 0;
	for(linePos = roomBeginning; linePos > -roomDepth; 
		linePos = linePos - (roomBeginning+roomDepth)/roomNrVLines) {
		// Lines right
		glBegin(GL_LINES);
			glVertex3f(roomSize/2, -roomSize/2, linePos);
			glVertex3f(roomSize/2, roomSize/2, linePos);
		glEnd();

		// Lines left
		glBegin(GL_LINES);
			glVertex3f(-roomSize/2, -roomSize/2, linePos);
			glVertex3f(-roomSize/2, roomSize/2, linePos);
		glEnd();

		// Lines bottom
		glBegin(GL_LINES);
			glVertex3f(-roomSize/2, -roomSize/2, linePos);
			glVertex3f(roomSize/2, -roomSize/2, linePos);
		glEnd();

		// Lines top
		glBegin(GL_LINES);
			glVertex3f(-roomSize/2, roomSize/2, linePos);
			glVertex3f(roomSize/2, roomSize/2, linePos);
		glEnd();
		
		glColor3f(1.0, 0.0, 0.0);
		drawQuad(randNumbersX[2*countX], randNumbersY[2*countY], linePos, 1.0);
		drawQuad(randNumbersX[2*countX+1], randNumbersY[2*countY+1], 
			linePos+0.5*((roomBeginning+roomDepth)/roomNrVLines), 1.0);
		countX = countX+1;
		countY = countY+1;
		glColor3f(1.0, 1.0, 1.0);
	}
	
	glutSwapBuffers();
}

void reshape (int w, int h)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45, 1.0 * w / h, 1, 10000);
  glViewport(0, 0, w, h);
  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0, 0, 100, 0, 0, 0, 0, 1, 0);	
}

void keyboard (unsigned char key, int x, int y) 
{ 
	switch (key) {
		case 'q':
		case 'Q':
			exit(EXIT_SUCCESS);
			break;
	}
}

void init(void)
{
  glClearColor (0.0, 0.0, 0.0, 0.0);
  
  glEnable(GL_DEPTH_TEST);

  glEnable (GL_FOG);
  glFogi (GL_FOG_MODE, GL_EXP);
  GLfloat density = 0.05;
  glFogf (GL_FOG_DENSITY, density);
  GLfloat fogColor[4] = {0.0, 0.0, 0.0, 1.0};
  glFogfv (GL_FOG_COLOR, fogColor);

  glEnable(GL_LINE_SMOOTH);
  glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  for(int i = 0; i < 200; i = i+1) {
  	randNumbersX[i] = (rand() % ((int) roomSize-1)) + 1 - (roomSize/2);
  	randNumbersY[i] = (rand() % ((int) roomSize-1)) + 1 - (roomSize/2);
  }
 }

int main(int argc,char** argv)
{	
	/* Setting up the Wiimotes */
	wiiht.connect();
	wiiht.setCameraPos(WIIHT_BELOW);
	wiiht.setSensitivity(4.5);
	wiiht.setEyeDist(730.0, 550.0);
	wiiht.setPosDist(10.0, 5.0);
	/* ----------------------- */
	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB);
	glutInitWindowSize (1024,768);
	glutInitWindowPosition (0, 0);
	glutCreateWindow ("Wiiheadtrack-Demo -- Object-Mode");
	init();
	
	glutKeyboardFunc(keyboard);
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
	glutIdleFunc(display);
	
	glutMainLoop();
	return 0;
}
