/* Wiiheadtrack-tool: eye-distance
 * 
 * This OpenGL-program is a tool for finding out the parameters distCenter 
 * and distNear of Wiiheadtrack's eyeDist() function. Follow the instructions of 
 * the program. Read the documentation for further information.
 * 
 * OpenGL must be installed on you system!
 * 
 * author:			Daniel Wunderlich (d.wunderlich@stud.uni-heidelberg.de)
 * author:			Christian Rohrbeck (c.rohrbec@stud.uni-heidelberg.de)
 * version:			1.0
 * last modified:	2010-07-30
 * created:			2010-07-30
 * web:				http://pille.iwr.uni-heidelberg.de/~tracking01/
 */

#include <iostream>
#include <cstdlib>
#include <GL/glut.h>
#include "wiiuse.h"

void init(void);
void display(void);
int main(int, char **);

wiimote** wiimotes;
unsigned int noWiimotes = 1;
unsigned int wiimoteCamNo = 0;
unsigned int foundWiimotes;
unsigned int connectedWiimotes;	
unsigned int scanTime = 5;

const char* headerLabel1 = "Tool for finding out the";
const char* headerLabel2 = "eye-distance for the Wiiheadtracker";

const char* helpLabelPress = "Press:";
const char* helpLabelC = "C for detecting distCenter";
const char* helpLabelN = "N for detecting distNear";
const char* helpLabelQ = "Q for quit";

float dist = 0.0;
float distCenter = 0.0;
float distNear = 0.0;
char distString[10];
char distCenterString[10];
char distNearString[10];

const char* distLabel = "current distance = ";
const char* distCenterLabel = "distCenter = ";
const char* distNearLabel = "distNear = ";

void textOutput(const char *str) 
{
	int i;

	for (i = 0; i < strlen(str); i++) {
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,str[i]);
	}
}

void display(void)
{
	if (wiiuse_poll(wiimotes, noWiimotes)) {
		if (wiimotes[wiimoteCamNo]->ir.z != 0) {
			dist = wiimotes[wiimoteCamNo]->ir.z;
		}
	}
	
	glClear(GL_COLOR_BUFFER_BIT);
		
	glColor3f (0.0,0.0,0.0);
	
	float textStart = -7.0;
	float lineskip = -1.0;
	float indent = 0.5;
	
	// Header
	
	float pos[2];
	pos[0] = textStart;
	pos[1] = 4.0;
	
	glRasterPos2f(pos[0], pos[1]);
	
	textOutput(headerLabel1);
	
	pos[1] += lineskip;
	
	glRasterPos2f(pos[0], pos[1]);
	
	textOutput(headerLabel2);
	
	// Distances
		
	if (dist == 0) {
		glColor3f(1.0, 0.0, 0.0);
	} else {
		glColor3f(0.0, 0.75, 0.0);
	}
	
	pos[0] += indent;
	pos[1] += 1.5*lineskip;
	glRasterPos2f(pos[0], pos[1]);
	
	sprintf(distString, "%.3f", dist);
	textOutput(distLabel);
	textOutput(distString);
	
	if (distCenter == 0) {
		glColor3f(1.0, 0.0, 0.0);
	} else {
		glColor3f(0.0, 0.75, 0.0);
	}
	
	pos[1] += lineskip;
	glRasterPos2f(pos[0], pos[1]);
	
	sprintf(distCenterString, "%.3f", distCenter);
	textOutput(distCenterLabel);
	textOutput(distCenterString);
	
	if (distNear == 0) {
		glColor3f(1.0, 0.0, 0.0);
	} else {
		glColor3f(0.0, 0.75, 0.0);
	}
	
	pos[1] += lineskip;
	glRasterPos2f(pos[0], pos[1]);
	
	sprintf(distNearString, "%.3f", distNear);
	textOutput(distNearLabel);
	textOutput(distNearString);
	
	// Help
	
	glColor3f(0.0, 0.0, 0.0);
	
	pos[0] = textStart;
	pos[1] += 2.0*lineskip;
	glRasterPos2f(pos[0], pos[1]);
	
	textOutput(helpLabelPress);
	
	pos[0] += indent;
	pos[1] += lineskip;
	glRasterPos2f(pos[0], pos[1]);
	textOutput(helpLabelC);
	
	pos[1] += lineskip;
	glRasterPos2f(pos[0], pos[1]);
	textOutput(helpLabelN);
	
	pos[1] += lineskip;
	glRasterPos2f(pos[0], pos[1]);
	textOutput(helpLabelQ);
	
	glutSwapBuffers();
}

void keyboard (unsigned char key, int x, int y) { 
		
	switch (key) {
		
		case 'q':
		case 'Q':
			printf ("Das Programm wird beendet.\n");
			exit(EXIT_SUCCESS);
			break;		
			
		case 'c':
		case 'C':
			distCenter = dist;
			break;
		
		case 'n':
		case 'N':
			distNear = dist;
			break;
	}
}

void init(void)
{
	glClearColor (1.0, 1.0, 1.0, 0.0);
	glEnable(GL_LINE_SMOOTH);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(-10.0, 10.0, -10.0, 10.0);
 }

int main(int argc,char** argv)
{	
	wiimotes =  wiiuse_init(noWiimotes);
	
	std::cout << "[Wiiheadtrack] Press 1 + 2 on the Wiimote!" 
		<< std::endl;
	
	foundWiimotes = wiiuse_find(wiimotes, noWiimotes, scanTime);
	
	if (!foundWiimotes) {
		std::cout <<"[Wiiheadtrack] Error: No Wiimote found!"
			<< std::endl;
	}
	
	connectedWiimotes = wiiuse_connect(wiimotes, noWiimotes); 
	
	if (connectedWiimotes) {
		std::cout << "[Wiiheadtrack] Connected to " 
			<< connectedWiimotes << " Wiimotes !" << std::endl;
				
		// Set the position of the Wiimote below the screen
		wiiuse_set_ir_position(wiimotes[wiimoteCamNo], 
			WIIUSE_IR_BELOW);
		
		// Enable IR-tracking
		wiiuse_set_ir(wiimotes[wiimoteCamNo], 1);
		
		// Set IR-sensitivity
		wiiuse_set_ir_sensitivity(wiimotes[wiimoteCamNo], 1); 
				
	} else { 
		std::cerr << "[Wiiheadtrack] Error: Failed to connect to "
			<< "any Wiimote." << std::endl;
		return 0;
	}
	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize (500, 500);
	glutInitWindowPosition (100, 100);
	glutCreateWindow ("eye-distance");
	init();
	
	glutKeyboardFunc(keyboard);
	glutDisplayFunc(display);
	glutIdleFunc(display);
	
	glutMainLoop();
	return 0;
}

